"""Модуль команд — реестр пользовательских команд."""

from cognitia.commands.registry import CommandDef, CommandHandler, CommandRegistry

__all__ = ["CommandDef", "CommandHandler", "CommandRegistry"]
